# $Id: simulate-version1.R 111 2009-03-05 10:49:30Z dreibh $
# ###########################################################################
#             Thomas Dreibholz's R Simulation Scripts Collection
#                  Copyright (C) 2005-2008 Thomas Dreibholz
#
#           Author: Thomas Dreibholz, dreibh@exp-math.uni-essen.de
# ###########################################################################
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Contact: dreibh@iem.uni-due.de


source("simulation.R")


# ###########################################################################
# #### demo-specific Part                                                ####
# ###########################################################################

# ====== Write OMNeT++ INI file header ======================================
demoWriteHeader <- function(iniFile, simulationRun, scalarName, vectorName, duration)
{
   cat(sep="", "[General]\n", file=iniFile)
   cat(sep="", "network = ", simCreatorSimulationNetwork, "\n", file=iniFile)
   cat(sep="", "rng-class = cMersenneTwister\n", file=iniFile)
   cat(sep="", "seed-0-mt = ", simulationRun, "\n", file=iniFile)
   cat(sep="", "output-scalar-file = ", "run", simulationRun, "-scalars.sca\n", file=iniFile)
   cat(sep="", "output-vector-file = ", "run", simulationRun, "-vectors.vec\n", file=iniFile)
   cat(sep="", "cmdenv-express-mode = true\n", file=iniFile)
   cat(sep="", "sim-time-limit = ", simCreatorSimulationStartup, " ", duration, "s 1ms\n", file=iniFile)
   cat(sep="", "\n\n", file=iniFile)

   if(simulationStoreVectors) {
      cat(sep="", "**.vector-recording = true\n", file=iniFile)
   }
   else {
      cat(sep="", "**.vector-recording = false\n", file=iniFile)
   }
   cat(sep="", "**.vector-recording-interval = ", simCreatorSimulationStartup, "..", simCreatorSimulationStartup, " ", duration, "s\n", file=iniFile)
   cat(sep="", "\n", file=iniFile)
   cat(sep="", "\n", file=iniFile)
}


# ====== demo automatic parameter setup =====================================
demoAutoParameters <- function(simulationConfigurations)
{
   return(TRUE)
}


# ====== Write demo INI file parameter section ==============================
demoWriteParameterSection <- function(filePrefix, iniFile, simulationRun, duration)
{
   # NewParam: Add appropriate lines to the .ini file in this function.

   # ------ Scenario settings -----------------------------------------------
   cat(sep="", "# ----- Scenario settings --------------------------------\n", file=iniFile)
   cat(sep="", "fragmenterScenario.fragmenter.cellPayloadSize = ", cellPayloadSize, "\n", file=iniFile)
   cat(sep="", "fragmenterScenario.fragmenter.cellHeaderSize = ", cellHeaderSize, "\n", file=iniFile)
   cat(sep="", "fragmenterScenario.intermediateNodeOutputRate = ", intermediateNodeOutputRate, "\n", file=iniFile)
   cat(sep="", "fragmenterScenario.sourceInterarrivalTime = exponential(", sourceInterarrivalTime, "s)\n", file=iniFile)
   cat(sep="", "fragmenterScenario.sourcePayloadSize = exponential(", sourcePayloadSize, ")\n", file=iniFile)
   cat(sep="", "fragmenterScenario.sourceHeaderSize = ", sourceHeaderSize, "\n", file=iniFile)
   cat(sep="", "\n", file=iniFile)
}


simCreatorSimulationBinary      <- "demo4"
simCreatorSimulationNetwork     <- "fragmenterScenario"
simCreatorSimulationStartup     <- "5s"
simCreatorWriteHeader           <- demoWriteHeader
simCreatorWriteParameterSection <- demoWriteParameterSection
simCreatorAutoParameters        <- demoAutoParameters
# Add variables here which should be handled as active (i.e. their value
# is recorded) even it is constant for all runs. The value of this variable
# is required for plotting.
# NewParam: If new parameters should always be recorded, add them here.
simCreatorAdditionalActiveVariables <- c(
   "cellPayloadSize",
   "cellHeaderSize",
   "sourceInterarrivalTime",
   "sourcePayloadSize",
   "sourceHeaderSize"
)

# Here, you can provide .ned files or directories containing .ned files. All
# these files will be copied into a new directory and transfered to the remote
# processing PEs. A corresponding nedfiles.lst will be written and also
# transferred to the PE.
# Examples: simCreatorNEDFiles  <- "."
#           simCreatorNEDFiles  <- "../alpha/nedfiles ../beta/nedfiles"
simCreatorNEDFiles  <- "."

# Here, you can provide additional files for packaging, e.g. "my-nedfile.ned".
# You may even use a pattern, e.g. "Test*.ned".
simulationMiscFiles <- ""



demoPlotVariables <- list(
   # ------ Format example --------------------------------------------------
   # list("Variable",
   #         "Unit[x]{v]"
   #          "100.0 * data1$x / data1$y", <- Manipulator expression:
   #                                           "data" is the data table
   #                                        NA here means: use data1$Variable.
   #          "myColor",
   #          list("InputFile1", "InputFile2", ...))
   #             (simulationDirectory/Results/....data.tar.bz2 is added!)
   # ------------------------------------------------------------------------

   list("cellPayloadSize",        "Cell Payload Size{P}[1]", NA, NA, NA),
   list("cellHeaderSize",         "Cell Header Size{H}[1]",  NA, NA, NA),
   list("sourceInterarrivalTime", "Interarrival Time{T}[s]", NA, NA, NA),
   list("sourcePayloadSize",      "Payload Size{p}[1]",      NA, NA, NA),
   list("sourceHeaderSize",       "Header Size{h}[1]",       NA, NA, NA),

   list("fragmenter.TotalPayload",
           "Total Payload in Fragmenter[1]",
           "100.0 * data1$fragmenter.TotalPayload",
           "green4",
           list("fragmenter-TotalPayload")),
   list("fragmenter.TotalOverhead",
           "Total Overhead in Fragmenter[1]",
           "100.0 * data1$fragmenter.TotalOverhead",
           "yellow4",
           list("fragmenter-TotalOverhead")),

   list("fragmenter.AddedOverhead",
           "Overhead Added by Fragmenter[%]",
           "100.0 * (data2$fragmenter.TotalOverhead / data1$fragmenter.TotalPayload)",
           "red4",
           list("fragmenter-TotalPayload", "fragmenter-TotalOverhead")),

   list("fragmenter.OverheadToPayloadRatio",
           "Overhead to Payload Ratio[%]",
           "100.0 * data1$fragmenter.OverheadToPayloadRatio",
           "brown4",
           list("fragmenter-OverheadToPayloadRatio")),

   list("sink", "Sink Number{S}", NA, NA, NA),

   list("sink.AverageDelay",
           "Average Delay[ms]",
           "data1$sink.AverageDelay * 1000.0",
           "blue4",
           list("sink-AverageDelay"))
)


# NewParam: Finally, add the new parameter to your default simulation configuration.

demoDefaultSimulationConfiguration <- list(
   # ------ Scenario Settings -----------------------------------------------
   list("cellPayloadSize", 128),
   list("cellHeaderSize", 4),
   list("intermediateNodeOutputRate", 150000),
   list("sourceInterarrivalTime", 0.1),
   list("sourcePayloadSize", 1000),
   list("sourceHeaderSize", 20)
)
